"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Debug = require("debug");
const debug = Debug("service:service");
const HttpServer_1 = require("./api/HttpServer");
const Metrics_1 = require("./Metrics");
const GoogleStorage_1 = require("./GoogleStorage");
const pjson = require("../package.json");
const TransactionProcessor_1 = require("./TransactionProcessor");
const GRACE_EXIT_MS = 1250;
class Service {
    constructor(config) {
        this.alive = true;
        this.ready = false;
        if (!config || typeof config !== "object") {
            throw new Error("Config must be an object: {kafka, http}");
        }
        if (!pjson || !pjson.name) {
            throw new Error("Missing package.json or package.json:name");
        }
        this.config = config;
        this.metrics = new Metrics_1.Metrics(pjson.name, [], []);
        this.httpServer = new HttpServer_1.default(this.config.http, this);
        this.googleStorage = new GoogleStorage_1.default(this.config.gcs);
        this.transactionProcess = new TransactionProcessor_1.default(this);
    }
    shutdownOnErrorIfNotProduction() {
        if (!Service.isProduction()) {
            debug("Shutting down (because of error) in", GRACE_EXIT_MS, "ms");
            this.close();
            setTimeout(() => {
                process.exit(1);
            }, GRACE_EXIT_MS);
        }
    }
    shutdownGracefully() {
        debug("\nShutting down gracefully in", GRACE_EXIT_MS, "ms");
        this.close();
        debug("Bye..");
        setTimeout(() => {
            process.exit(0);
        }, GRACE_EXIT_MS);
    }
    init() {
        process.on("SIGINT", this.shutdownGracefully.bind(this));
        process.on("SIGUSR1", this.shutdownGracefully.bind(this));
        process.on("SIGUSR2", this.shutdownGracefully.bind(this));
        /*
        process.on("warning", (warning: Error) => {
            debug("Warning:", warning.message);
        }); */
        process.on("uncaughtException", (error) => {
            debug("Unhandled Exception: ", error.message, error.stack);
            this.shutdownOnErrorIfNotProduction();
        });
        process.on("unhandledRejection", (reason, promise) => {
            debug("Unhandled Rejection: ", reason);
            this.shutdownOnErrorIfNotProduction();
        });
        if (Service.isProduction()) {
            debug("Running production.");
        }
        else {
            debug("Running NOT in production.");
        }
    }
    async run() {
        this.init();
        debug("Starting..");
        this.metrics.registerDefault();
        await this.httpServer.start();
        await this.googleStorage.init();
        this.setReadyState(true);
        debug("Running..");
    }
    async close() {
        debug("Closing..");
        this.setAliveState(false);
        this.setReadyState(false);
        this.httpServer.close();
        this.transactionProcess.close();
        this.metrics.close();
    }
    static isProduction() {
        return process.env.NODE_ENV === "production";
    }
    setAliveState(state) {
        debug("Setting alive state from", this.alive, "to", state);
        this.alive = state;
    }
    isAlive() {
        return this.alive;
    }
    getHealthInfo() {
        const alive = this.isAlive();
        const status = alive ? 200 : 503;
        return [status, {
                status: alive ? "UP" : "DOWN",
            }];
    }
    setReadyState(state) {
        debug("Setting ready state from", this.ready, "to", state);
        this.ready = state;
    }
    isReady() {
        return this.ready;
    }
}
exports.default = Service;
