"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Debug = require("debug");
const debug = Debug("service:processor");
const Bluebird = require("bluebird");
const CustomStream_1 = require("./CustomStream");
const HttpClient_1 = require("./HttpClient");
class TransactionProcessor {
    constructor(service) {
        this.service = service;
        this.transactionsInProcess = [];
        this.intv = setInterval(() => {
            this.service.metrics.set("current_transactions", this.transactionsInProcess.length);
        }, 500);
    }
    async processTransaction(transactionId, chunks, baseUrl = "http://localhost:8080", contentType = "application/octet-stream", extension = "parquet") {
        if (!baseUrl) {
            throw new Error("baseUrl is missing.");
        }
        const chunkMap = [];
        for (let i = 0; i < chunks; i++) {
            chunkMap.push(i);
        }
        if (!chunkMap.length) {
            throw new Error("No chunks to process.");
        }
        if (this.getInProcess(transactionId)) {
            throw new Error("transactionId " + transactionId + " is already in process.");
        }
        const transaction = {
            transactionId,
            chunks,
            baseUrl,
            contentType,
            extension,
            atChunk: 0,
        };
        this.addInProcess(transaction);
        if (!(await this.startTransaction(baseUrl, transactionId))) {
            this.removeInProcess(transactionId, false);
            throw new Error("Could not start transaction");
        }
        const gcs$ = this.service.googleStorage.getStoreStream(`${transactionId}.${extension}`, contentType);
        // no need to await, because gcs$.end() controlls gcs$.promise resolving
        Bluebird.mapSeries(chunkMap, async (chunkId) => {
            this.setInProcessChunk(transactionId, chunkId);
            await this.streamChunk(gcs$, transactionId, chunkId, baseUrl);
        }).then(() => {
            gcs$.end();
        });
        return gcs$.promise.then(() => {
            debug("Transaction streaming done.");
            return this.ackTransaction(baseUrl, transactionId).then((result) => {
                this.removeInProcess(transactionId, true);
                return result;
            }).catch((error) => {
                this.removeInProcess(transactionId, false);
                throw error;
            });
        }).catch((error) => {
            debug("Transaction failed", error.message);
            this.removeInProcess(transactionId, false);
            return false;
        });
    }
    getTransactionsInProcess() {
        return this.transactionsInProcess;
    }
    close() {
        if (this.intv) {
            clearInterval(this.intv);
        }
    }
    streamChunk(gcs$, transactionId, chunkId, baseUrl) {
        return new Promise((resolve) => {
            const custom$ = new CustomStream_1.default(gcs$, true);
            const http$ = HttpClient_1.default.getRequestStream({
                url: `${baseUrl}/request/${transactionId}/chunk/${chunkId}`,
                method: "GET",
            });
            http$.pipe(custom$);
            http$.on("end", () => {
                resolve();
            });
        });
    }
    async startTransaction(baseUrl, transactionId) {
        const { status, body } = await HttpClient_1.default.call({
            url: `${baseUrl}/start/${transactionId}`,
            method: "POST",
        });
        debug("Start transation", status);
        this.service.metrics.inc("transaction_start");
        return status === 200;
    }
    async ackTransaction(baseUrl, transactionId) {
        const { status, body } = await HttpClient_1.default.call({
            url: `${baseUrl}/ack/${transactionId}`,
            method: "DELETE",
        });
        debug("ACK transation", status);
        this.service.metrics.inc("transaction_ack");
        return status === 204;
    }
    addInProcess(transaction) {
        this.service.metrics.inc("transaction_start");
        this.transactionsInProcess.push(transaction);
    }
    getInProcess(transactionId) {
        for (let i = this.transactionsInProcess.length - 1; i >= 0; i--) {
            if (this.transactionsInProcess[i].transactionId === transactionId) {
                return this.transactionsInProcess[i];
            }
        }
        return null;
    }
    setInProcessChunk(transactionId, chunkId) {
        this.service.metrics.inc("transaction_chunk");
        const transaction = this.getInProcess(transactionId);
        if (transaction) {
            debug("Updating transaction chunk", transactionId, transaction.atChunk, "->", chunkId);
            transaction.atChunk = chunkId;
            return true;
        }
        return false;
    }
    removeInProcess(transactionId, successfull) {
        if (successfull) {
            this.service.metrics.inc("transaction_successfull");
        }
        else {
            this.service.metrics.inc("transaction_failed");
        }
        this.service.metrics.inc("transaction_done");
        for (let i = this.transactionsInProcess.length - 1; i >= 0; i--) {
            if (this.transactionsInProcess[i].transactionId === transactionId) {
                this.transactionsInProcess.splice(i, 1);
                return true;
            }
        }
        return false;
    }
}
exports.default = TransactionProcessor;
